#include <iostream>
#include <vector>
#include <algorithm>
using namespace std;

typedef void (sort_func)(vector<wstring>&);

wostream& operator <<(wostream& wstr, vector<wstring>& v) { // отладка-контроль
   wstr << L"[ ";
   for (auto x : v) wstr << x << L" ";
   return wstr << L"]";
}

int main(int argc, char *argv[]) {
   locale::global(locale(""));
   // предварительные объявления функций сортировки:
   sort_func sort1, sort2, sort3, sort4, sort5, sort6;
   sort_func* variants[] = {
      sort1, sort2, sort3, sort4, sort5, sort6
   };
   vector<wstring> vect = { 
      L"Фёдоров", L"Петров", L"Сидоров", L"Иванов", L"Чапаев", L"Фурманов"
   }; 
   wcout << vect << endl 
         << L"-------------------------------------------------" << endl;
   for (unsigned i = 0; i < sizeof(variants) / sizeof(variants[0]); i++) {
      vector<wstring> vcopy(vect);
      variants[i](vcopy);
      wcout << vcopy << endl;
   } 
}

//--------------------------------------------------------------------------------------------
// Быстрая сортировка. Сложность в среднем O(n*log(n)), но в худшем случае O(n*n)
void sort1(vector<wstring>& v) {
   sort(v.begin(), v.end());
}
//--------------------------------------------------------------------------------------------
bool sort_function(wstring& f, wstring& s) { return f < s; }

void sort2(vector<wstring>& v) { // использование функции как предикакта
   sort(v.begin(), v.end(), sort_function);
}
//--------------------------------------------------------------------------------------------
struct sort_class {   // функтор сравнения
   bool operator()(wstring& f, wstring& s) { return f < s; }
};

void sort3(vector<wstring>& v) {
   sort(v.begin(), v.end(), sort_class());
}
//--------------------------------------------------------------------------------------------
// Сортировка подпоследователности. Гарантированная сложность O(n*log(n)) в любом случае.
// Обычно сортировка в куче выполняется в 2-5 раз медленнее быстрой сортировки sort(). 
void sort4(vector<wstring>& v) {
   partial_sort(v.begin(), v.end(), v.end());
}
//--------------------------------------------------------------------------------------------
// Сортировка слиянием. Сложность O(n*log(n)) или O(n*log(n)*log(n)), если без дополнительной памяти 
void sort5(vector<wstring>& v) {
   stable_sort(v.begin(), v.end());
}
//--------------------------------------------------------------------------------------------
// Сортировка в куче (heap) - вызывают функции, непосредственно работающие с кучей 
// (то есть с бинарным деревом, используемым в реализации этих алгоритмов). 
// Сложность O(n+n*log(n))
void sort6(vector<wstring>& v) {
   make_heap(v.begin(), v.end());
   sort_heap(v.begin(), v.end());
}
//--------------------------------------------------------------------------------------------
